/* Copyright (C) 1991, 2000 Aladdin Enterprises.  All rights reserved.
  
  This file is part of AFPL Ghostscript.
  
  AFPL Ghostscript is distributed with NO WARRANTY OF ANY KIND.  No author or
  distributor accepts any responsibility for the consequences of using it, or
  for whether it serves any particular purpose or works at all, unless he or
  she says so in writing.  Refer to the Aladdin Free Public License (the
  "License") for full details.
  
  Every copy of AFPL Ghostscript must include a copy of the License, normally
  in a plain ASCII text file named PUBLIC.  The License grants you the right
  to copy, modify and redistribute AFPL Ghostscript, but only under certain
  conditions described in the License.  Among other things, the License
  requires that the copyright notice and this notice be preserved on all
  copies.
*/

/*$Id: ifontapi.h,v 1.0 2000/12/03 23:35:30 igorm Exp $ */
/* Font API interface */

#include "iapi.h"
#include "gsplugin.h"

typedef int FracInt;

typedef int FAPI_retcode;

typedef struct FAPI_path_interface_s FAPI_path_interface;
struct gx_path_s;

struct FAPI_path_interface_s {
    struct FAPI_outline_handler_s *olh;
    int shift;
    int (*moveto)(FAPI_path_interface *, FracInt, FracInt);
    int (*lineto)(FAPI_path_interface *, FracInt, FracInt);
    int (*curveto)(FAPI_path_interface *, FracInt, FracInt, FracInt, FracInt, FracInt, FracInt);
    int (*closepath)(FAPI_path_interface *);
};

typedef struct FAPI_char_ref_s {
    int char_code;
    const unsigned char *char_name; /* to be used exclusively with char_code. */
    unsigned int char_name_length;
} FAPI_char_ref;

typedef struct FAPI_raster_s { /* 1bit/pixel only, lines are byte-aligned. */
    void *p;
    int width, height, line_step;
} FAPI_raster;

typedef struct FAPI_metrics_s {
    int bbox_x0, bbox_y0, bbox_x1, bbox_y1; /* pixels */
    int orig_x, orig_y; /* origin, 1/16s pixel */
    int lsb_x, lsb_y; /* left side bearing */
    int tsb_x, tsb_y; /* top side bearing */
    FracInt esc_x, esc_y; /* escapement */
} FAPI_metrics;

typedef struct FAPI_interface_s {
    gs_plugin_interface ig;
    int frac_shift; /* number of fractional bits in coordinates */
    int renderer_caches_chars; /* boolean */
    FAPI_retcode (*ensure_open)(gs_plugin_interface *renderer);
    FAPI_retcode (*get_font)(gs_plugin_interface *renderer, const char *font_file_path, int subfont, const FracInt matrix[6], const FracInt HWResolution[2], const char *xlatmap, int bCache, void **font_handle);
    FAPI_retcode (*release_font)(gs_plugin_interface *renderer, void *font_handle);
    FAPI_retcode (*get_decodingID)(gs_plugin_interface *renderer, void *font_handle, const char **decodingID);
    FAPI_retcode (*get_font_bbox)(gs_plugin_interface *renderer, void *font_handle, int BBox[4]);
    FAPI_retcode (*can_retrive_char_by_name)(gs_plugin_interface *renderer, void *font_handle, int *result);
    FAPI_retcode (*outline_char)(gs_plugin_interface *renderer, void *font_handle, FAPI_char_ref *c, FAPI_path_interface *p, FAPI_metrics *metrics);
    FAPI_retcode (*get_char_width)(gs_plugin_interface *renderer, void *font_handle, FAPI_char_ref *c, FracInt *wx, FracInt *wy);
    FAPI_retcode (*get_char_raster_metrics)(gs_plugin_interface *renderer, void *font_handle, FAPI_char_ref *c, FAPI_metrics *metrics);
    FAPI_retcode (*get_char_raster)(gs_plugin_interface *renderer, void *font_handle, FAPI_char_ref *c, FAPI_raster *r);
    FAPI_retcode (*release_char_raster)(gs_plugin_interface *renderer, void *font_handle, FAPI_char_ref *c);
    /*  get_char_raster_metrics may leave some data kept by the renderer,
	so as get_char_raster uses them and release_char_raster releases them.
	In current version calls from GS to these functions must not interfer with 
	different characters, to provide compatibility with both GS caching 
	and renderer caching - to be improved later.

	If GS caches characters, FAPI_raster parameter passes information 
	from GS to renderer. Othervise it can do backwards - reserved for future.

        Currently GS cannot provide information when a font to be closed.
        Therefore we don't provide close_font function in this API.
        The renderer must cache fonts, and close ones which were
        not in use for a long time. Of cause, it must close all fonts
        on close_renderer.
    */
} FAPI_interface;
